@@ -458,8 +458,10 @@ Example:
 
 =item dkim_timeout n             (default: 5)
 
-How many seconds to wait for a DKIM query to complete, before
-scanning continues without the DKIM result.
+How many seconds to wait for a DKIM query to complete, before scanning
+continues without the DKIM result. A numeric value is optionally suffixed
+by a time unit (s, m, h, d, w, indicating seconds (default), minutes, hours,
+days, weeks).
 
 =back
 
@@ -469,7 +471,7 @@ scanning continues without the DKIM result.
     setting => 'dkim_timeout',
     is_admin => 1,
     default => 5,
-    type => $Mail::SpamAssassin::Conf::CONF_TYPE_NUMERIC
+    type => $Mail::SpamAssassin::Conf::CONF_TYPE_DURATION
   });
 
   $conf->{parser}->register_commands(\@cmds);
@@ -663,7 +665,9 @@ sub _check_dkim_signed_by {
       next if $sig->UNIVERSAL::can("check_expiration") &&
               !$sig->check_expiration;
     }
-    my $sdid = lc($sig->domain);
+    my $sdid = $sig->domain;
+    next if !defined $sdid;  # a signature with a missing required tag 'd' ?
+    $sdid = lc $sdid;
     if ($must_be_author_domain_signature) {
       next if !$pms->{dkim_author_domains}->{$sdid};
     }
@@ -742,7 +746,18 @@ sub _check_dkim_signature {
     # signature objects not provided by the caller, must verify for ourselves
     my $timemethod = $self->{main}->UNIVERSAL::can("time_method") &&
                      $self->{main}->time_method("check_dkim_signature");
-    $verifier = Mail::DKIM::Verifier->new();
+    if (Mail::DKIM::Verifier->VERSION >= 0.40) {
+      my $edns = $pms->{conf}->{dns_options}->{edns};
+      if ($edns && $edns >= 1024) {
+        # Let Mail::DKIM use our interface to Net::DNS::Resolver.
+        # Only do so if EDNS0 provides a reasonably-sized UDP payload size,
+        # as our interface does not provide a DNS fallback to TCP, unlike
+        # the Net::DNS::Resolver::send which does provide it.
+        my $res = $self->{main}->{resolver}->get_resolver;
+        Mail::DKIM::DNS::resolver($res);
+      }
+    }
+    $verifier = Mail::DKIM::Verifier->new;
     if (!$verifier) {
       dbg("dkim: cannot create Mail::DKIM::Verifier object");
       return;
@@ -812,14 +827,20 @@ sub _check_dkim_signature {
       }
       push(@valid_signatures, $signature)  if $valid && !$expired;
       # check if we have a potential Author Domain Signature, valid or not
-      my $d = lc($signature->domain);
-      if ($pms->{dkim_author_domains}->{$d}) {  # SDID matches author domain
-        $pms->{dkim_has_any_author_sig}->{$d} = 1;
-        if ($valid && !$expired) {
-          $pms->{dkim_has_valid_author_sig}->{$d} = 1;
-        } elsif ( ($sig_result_supported ?$signature :$verifier)->result_detail
-                 =~ /\b(?:timed out|SERVFAIL)\b/i) {
-          $pms->{dkim_author_sig_tempfailed}->{$d} = 1;
+      my $d = $signature->domain;
+      if (!defined $d) {
+        # can be undefined on a broken signatures with missing required tags
+      } else {
+        $d = lc $d;
+        if ($pms->{dkim_author_domains}->{$d}) {  # SDID matches author domain
+          $pms->{dkim_has_any_author_sig}->{$d} = 1;
+          if ($valid && !$expired) {
+            $pms->{dkim_has_valid_author_sig}->{$d} = 1;
+          } elsif ( ($sig_result_supported ? $signature
+                                           : $verifier)->result_detail
+                   =~ /\b(?:timed out|SERVFAIL)\b/i) {
+            $pms->{dkim_author_sig_tempfailed}->{$d} = 1;
+          }
         }
       }
       if (would_log("dbg","dkim")) {
@@ -830,8 +851,10 @@ sub _check_dkim_signature {
           $signature->algorithm, scalar($signature->canonicalization),
           ($sig_result_supported ? $signature : $verifier)->result,
           !$expired ? '' : ', expired',
-          $pms->{dkim_author_domains}->{$d} ? 'matches author domain' :
-                                              'does not match author domain');
+          defined $d && $pms->{dkim_author_domains}->{$d}
+            ? 'matches author domain'
+            : 'does not match author domain'
+        );
       }
     }
     if (@valid_signatures) {
@@ -841,13 +864,17 @@ sub _check_dkim_signature {
       my $sig = $valid_signatures[0];
       my $sig_res = ($sig_result_supported ? $sig : $verifier)->result_detail;
       dbg("dkim: signature verification result: %s", uc($sig_res));
-      my(%seen1,%seen2);
+
+      # supply values for both tags
+      my(%seen1, %seen2, @identity_list, @domain_list);
+      @identity_list = grep(defined $_ && $_ ne '' && !$seen1{$_}++,
+                            map($_->identity, @valid_signatures));
+      @domain_list =   grep(defined $_ && $_ ne '' && !$seen2{$_}++,
+                            map($_->domain, @valid_signatures));
       $pms->set_tag('DKIMIDENTITY',
-              join(" ", grep { defined($_) && $_ ne '' && !$seen1{$_}++ }
-                         map { $_->identity } @valid_signatures));
+                    @identity_list == 1 ? $identity_list[0] : \@identity_list);
       $pms->set_tag('DKIMDOMAIN',
-              join(" ", grep { defined($_) && $_ ne '' && !$seen2{$_}++ }
-                         map { $_->domain } @valid_signatures));
+                    @domain_list == 1   ? $domain_list[0]   : \@domain_list);
     } elsif (@signatures) {
       $pms->{dkim_signed} = 1;
       my $sig = $signatures[0];
@@ -966,8 +993,11 @@ sub _check_dkim_adsp {
               if (Mail::DKIM::AuthorDomainPolicy->UNIVERSAL::can("fetch")) {
                 dbg("dkim: adsp: performing lookup on _adsp._domainkey.%s",
                     $author_domain);
+                # get our Net::DNS::Resolver object
+                my $res = $self->{main}->{resolver}->get_resolver;
                 $practices = Mail::DKIM::AuthorDomainPolicy->fetch(
-                               Protocol => "dns", Domain => $author_domain);
+                               Protocol => "dns", Domain => $author_domain,
+                               DnsResolver => $res);
               }
               1;
             } or do {
@@ -1084,6 +1114,8 @@ sub _wlcheck_acceptable_signature {
     foreach my $white_addr (keys %$wl_ref) {
       my $wl_addr_ref = $wl_ref->{$white_addr};
       my $re = qr/$wl_addr_ref->{re}/i;
+    # dbg("dkim: WL %s %s, d: %s", $wl, $white_addr,
+    #     join(", ", map { $_ eq '' ? "''" : $_ } @{$wl_addr_ref->{domain}}));
       if ($author =~ $re) {
         foreach my $sdid (@{$wl_addr_ref->{domain}}) {
           push(@$acceptable_sdid_tuples_ref, [$author,$sdid,$wl,$re]);
@@ -1103,6 +1135,7 @@ sub _wlcheck_author_signature {
   foreach my $author (@{$pms->{dkim_author_addresses}}) {
     foreach my $white_addr (keys %$wl_ref) {
       my $re = $wl_ref->{$white_addr};
+    # dbg("dkim: WL %s %s", $wl, $white_addr);
       if ($author =~ $re) {
         push(@$acceptable_sdid_tuples_ref, [$author,undef,$wl,$re]);
       }
@@ -1128,7 +1161,8 @@ sub _wlcheck_list {
     if ($valid && $signature->UNIVERSAL::can("check_expiration")) {
       $expired = !$signature->check_expiration;
     }
-    my $sdid = lc($signature->domain);
+    my $sdid = $signature->domain;
+    $sdid = lc $sdid  if defined $sdid;
 
     my $info = $valid ? 'VALID' : 'FAILED';
     $info .= ' EXPIRED'  if $expired;
@@ -1147,8 +1181,10 @@ sub _wlcheck_list {
       $tried_authors{$author} = 1;  # for logging purposes
 
       my $matches = 0;
-      if (!defined $acceptable_sdid || $acceptable_sdid eq '') {
+      if (!defined $sdid) {
+        # don't bother, invalid signature with a missing 'd' tag
 
+      } elsif (!defined $acceptable_sdid || $acceptable_sdid eq '') {
         # An "Author Domain Signature" (sometimes called a first-party
         # signature) is a Valid Signature in which the domain name of the
         # DKIM signing entity, i.e., the d= tag in the DKIM-Signature header
@@ -1157,9 +1193,8 @@ sub _wlcheck_list {
 
         # checking for Author Domain Signature
         $matches = 1  if $sdid eq $author_domain;
-      }
-      else {  # checking for verifier-acceptable signature
 
+      } else {  # checking for verifier-acceptable signature
         # The second argument to a 'whitelist_from_dkim' option is now (since
         # version 3.3.0) supposed to be a signing domain (SDID), no longer an
         # identity (AUID). Nevertheless, be prepared to accept the full e-mail
@@ -1191,7 +1226,8 @@ sub _wlcheck_list {
       }
     }
     dbg("dkim: %s signature by %s, author %s, no valid matches",
-        $info, $sdid, join(", ", keys %tried_authors))  if !$any_match_at_all;
+        $info,  defined $sdid ? $sdid : '(undef)',
+        join(", ", keys %tried_authors))  if !$any_match_at_all;
   }
   return ($any_match_at_all, \%any_match_by_wl);
 }
